/*
 * Copyright 2017-2020 original authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.micronaut.inject;

import io.micronaut.core.annotation.NonNull;
import io.micronaut.core.beans.BeanConstructor;

/**
 * A constructor injection point.
 *
 * @param <T> The constructed type
 * @author Graeme Rocher
 * @since 1.0
 */
public interface ConstructorInjectionPoint<T> extends CallableInjectionPoint<T>, BeanConstructor<T> {

    /**
     * Invoke the constructor.
     *
     * @param args The arguments
     * @return The new value
     */
    T invoke(Object... args);

    @Override
    default @NonNull Class<T> getDeclaringBeanType() {
        return getDeclaringBean().getBeanType();
    }

    @Override
    default @NonNull T instantiate(Object... parameterValues) {
        return invoke(parameterValues);
    }
}
